-------------------------------------------------------------------------------
--                               Lock Manager                                --
--                                                                           --
--  This is the root type for the lock manager                               --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2001/02/12 14:15:31 $
--  $RCSfile: lock_managers.ads,v $
--  $Revision: 1.7 $
--                                                                           --
-------------------------------------------------------------------------------

with Locks; use Locks;
with Operations.Normal; use Operations.Normal;
with Transactions; use Transactions;

with Ada.Finalization;

package Lock_Managers is

   type Lock_Manager_Type is tagged limited private;
   type Lock_Manager_Ref is access all Lock_Manager_Type'Class;

   procedure Request_Lock_And_Mutex (Lock_Manager : Lock_Manager_Type;
                                     Operation : Normal_Operation_Type'Class;
                                     Transaction : Transaction_Ref);

   procedure Free_Mutex (Lock_Manager : Lock_Manager_Type);

   procedure Transaction_Commit (Lock_Manager : Lock_Manager_Type;
                                 Transaction : Transaction_Ref);

   procedure Transaction_Abort (Lock_Manager : Lock_Manager_Type;
                                Transaction : Transaction_Ref);

private

   type Lock_Node_Type;

   type Lock_Node_Ref is access Lock_Node_Type;

   type Lock_Node_Type is record
      Lock : Lock_Ref;
      Transaction : Transaction_Ref;
      Next : Lock_Node_Ref;
   end record;

   type Lock_Set_Type is record
      Head : Lock_Node_Ref := null;
   end record;

   protected type Protected_Lock_Manager_Type is

      --  The following two operations are called by the object proxy
      entry Request_Lock_And_Mutex (Lock : Lock_Ref;
                                    Transaction : Transaction_Ref);

      procedure Free_Mutex;

      --  The following two operations are called by the recovery manager
      procedure Transaction_Commit (Transaction : Transaction_Ref);

      procedure Transaction_Abort (Transaction : Transaction_Ref);

   private

      entry Waiting_For_Lock (Lock : Lock_Ref;
                              Transaction : Transaction_Ref);

      entry Waiting_For_Writer (Lock : Lock_Ref;
                                Transaction : Transaction_Ref);

      entry Waiting_For_Readers (Lock : Lock_Ref;
                                 Transaction : Transaction_Ref);

      --  These two variables implement mutual exclusion among tasks of the
      --  same transaction. Operations that only read the data of the object
      --  are allowed to execute concurrently.
      Currently_Writing : Boolean := False;
      Currently_Reading : Natural := 0;

      To_Try : Natural := 0;

      My_Locks : Lock_Set_Type;

   end Protected_Lock_Manager_Type;

   type Protected_Lock_Manager_Ref is access Protected_Lock_Manager_Type;

   type Lock_Manager_Type is new Ada.Finalization.Limited_Controlled
   with record
      My_Lock_Manager : Protected_Lock_Manager_Ref := null;
   end record;

   procedure Initialize (Manager : in out Lock_Manager_Type);

   procedure Finalize (Manager : in out Lock_Manager_Type);

end Lock_Managers;



