-------------------------------------------------------------------------------
--                                Cache Manager                              --
--                                                                           --
--  This is the root type for any cache manager.                             --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/09/29 07:48:42 $
--  $RCSfile: cache_managers.adb,v $
--  $Revision: 1.9 $
--                                                                           --
-------------------------------------------------------------------------------

with Memory_Objects.Concrete; use Memory_Objects.Concrete;
with Memory_Objects.Concrete.Undo_Redoable;
use Memory_Objects.Concrete.Undo_Redoable;

with Logging_Techniques.Physical; use Logging_Techniques.Physical;
with Logging_Techniques.Logical; use Logging_Techniques.Logical;

with Debug; use Debug;

with Recovery_Support; use Recovery_Support;
with Recovery_Managers; use Recovery_Managers;

package body Cache_Managers is

   pragma Warnings (Off);
   Logical_Factory : aliased Logical_Factory_Type;
   Physical_Factory : aliased Physical_Factory_Type;
   pragma Warnings (On);

   Memory_Object_List : Memory_Object_Lists.List_Type;

   use Memory_Object_Lists;

   procedure Flush (M : in out Cache_Manager_Type) is
   begin
      null;
   end Flush;

   procedure Flush_All (M : in out Cache_Manager_Type) is
   begin
      null;
   end Flush_All;

   function Create (Cache    : in Cache_Manager_Type;
                    Params   : Non_Volatile_Params_Type'Class;
                    Creation : Creation_Operation_Type'Class;
                    Loading  : Loading_Operation_Type'Class;
                    Saving   : Saving_Operation_Type'Class;
                    Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref is
      Tmp : Element_Node_Ref;
   begin
      Tmp := Find (Memory_Object_List, Params);
      raise Object_Exists;
   exception
      when No_Such_Key =>
         declare
            New_Object : Memory_Object_Ref := Memory_Object_Ref
           (Memory_Objects.Concrete.Undo_Redoable.Create
            (Self (Cache), Logical_Factory'Access));
            New_Params : Non_Volatile_Params_Ref :=
              new Non_Volatile_Params_Type'Class' (Params);
         begin
            Insert_Element_With_Info (Memory_Object_List,
               New_Params, New_Object, False, True);
            Create (Memory_Object_Type'Class (New_Object.all),
                    Params,
                    Creation,
                    Loading,
                    Saving,
                    Deletion);
            return New_Object;
         end;
   end Create;

   function Restore (Cache    : Cache_Manager_Type;
                     Params   : Non_Volatile_Params_Type'Class;
                     Loading  : Loading_Operation_Type'Class;
                     Saving   : Saving_Operation_Type'Class;
                     Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref is
      Tmp : Element_Node_Ref;
      Recovery_Needed : Boolean := False;
   begin
      Put_Line ("Cache: In restore");
      begin
         Recovery_Needed := Get_Info (Memory_Object_List, Params);
         Tmp := Find (Memory_Object_List, Params);
         if Tmp /= null then
            return Tmp.Element;
         end if;
      exception
         when No_Such_Key =>
            null;
      end;

      declare
         New_Object : Memory_Object_Ref := Memory_Object_Ref
           (Memory_Objects.Concrete.Undo_Redoable.Create
            (Self (Cache), Logical_Factory'Access));
         New_Params : Non_Volatile_Params_Ref :=
          new Non_Volatile_Params_Type'Class' (Params);
      begin
         Restore (Memory_Object_Type'Class (New_Object.all),
                  Params,
                  Loading,
                  Saving,
                  Deletion);
         if Recovery_Needed then
            Recover_Object (Get_Recovery_Manager.all,
                            Concrete_Memory_Object_Ref (New_Object),
                            New_Params,
                            Loading);
         end if;
         Insert_Element_With_Info (Memory_Object_List,
                                   New_Params, New_Object, False, True);
         Put_Line ("Cache: Inserted new object");
         return New_Object;
      end;
   end Restore;

   procedure Tag_Object (Cache : in out Cache_Manager_Type;
                         Params : Non_Volatile_Params_Type'Class) is
   begin
      Put_Line ("Cache: Tag object called");
      declare
         New_Params : Non_Volatile_Params_Ref :=
           new Non_Volatile_Params_Type'Class' (Params);
      begin
         Create_Key (Memory_Object_List, New_Params, True);
      end;
   end Tag_Object;

   function Self (M : in Cache_Manager_Type) return Cache_Manager_Ref is
   begin
      return M.Self.Obj.all'Unchecked_Access;
   end Self;

end Cache_Managers;
