-------------------------------------------------------------------------------
--                                Cache Manager                              --
--                                                                           --
--  This is the root type for any cache manager.                             --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/09/29 07:48:43 $
--  $RCSfile: cache_managers.ads,v $
--  $Revision: 1.8 $
--                                                                           --
-------------------------------------------------------------------------------

with Memory_Objects; use Memory_Objects;
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;
with Operations.Creation; use Operations.Creation;
with Operations.Loading; use Operations.Loading;
with Operations.Saving; use Operations.Saving;
with Operations.Deletion; use Operations.Deletion;

with Refkeyed_Lists;

package Cache_Managers is

   type Cache_Manager_Type is abstract tagged limited private;

   type Cache_Manager_Ref is access all Cache_Manager_Type'Class;

   procedure Flush (M : in out Cache_Manager_Type);

   procedure Flush_All (M : in out Cache_Manager_Type);

   function Create (Cache    : in Cache_Manager_Type;
                    Params   : Non_Volatile_Params_Type'Class;
                    Creation : Creation_Operation_Type'Class;
                    Loading  : Loading_Operation_Type'Class;
                    Saving   : Saving_Operation_Type'Class;
                    Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref;

   function Restore (Cache    : in Cache_Manager_Type;
                     Params   : Non_Volatile_Params_Type'Class;
                     Loading  : Loading_Operation_Type'Class;
                     Saving   : Saving_Operation_Type'Class;
                     Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref;

   procedure Apply_Replacement_Policy (M : in out Cache_Manager_Type)
     is abstract;

   procedure Tag_Object (Cache : in out Cache_Manager_Type;
                         Params : Non_Volatile_Params_Type'Class);

private

   package Memory_Object_Lists is new Refkeyed_Lists
     (Non_Volatile_Params_Type,
      Non_Volatile_Params_Ref, Memory_Object_Ref, Boolean);

   type Selfer (Obj : access Cache_Manager_Type'Class) is
      limited null record;

   type Cache_Manager_Type is abstract tagged limited record
      Self  : Selfer (Cache_Manager_Type'Access);
   end record;

   function Self (M : in Cache_Manager_Type) return Cache_Manager_Ref;

end Cache_Managers;
