-------------------------------------------------------------------------------
--                   Memory Objects.Concrete.Undo_Redoable                   --
--                                                                           --
--  This type implements the versioned memory objects.                       --
--  It uses a stack of memory storage to store the different states of       --
--  a data object                                                            --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2001/04/24 07:06:52 $
--  $RCSfile: memory_objects-concrete-undo_redoable.ads,v $
--  $Revision: 1.4 $
--                                                                           --
-------------------------------------------------------------------------------

with Cache_Managers; use Cache_Managers;

with Logging_Techniques; use Logging_Techniques;

with Transactions; use Transactions;

with Linked_Lists;
with Keyed_Lists;

package Memory_Objects.Concrete.Undo_Redoable is

   type Undo_Redoable_Object_Type is new Memory_Object_Type with private;
   type Undo_Redoable_Object_Ref is access all Undo_Redoable_Object_Type'Class;

   function Create (Cache    : Cache_Manager_Ref;
                    Logging_Factory : Logging_Factory_Ref)
      return Undo_Redoable_Object_Ref;

   procedure Execute (Object      : in out Undo_Redoable_Object_Type;
                      Operation   : in out Normal_Operation_Type'Class;
                      Transaction : in Transaction_Ref);

   function Get_Undo_List (Object : Undo_Redoable_Object_Type;
                           Transaction : Transaction_Ref) return
     Logging_Technique_Array_Type;

   function Get_Redo_List (Object : Undo_Redoable_Object_Type;
                           Transaction : Transaction_Ref) return
     Logging_Technique_Array_Type;

   Transaction_Not_Found : exception;

private

   package Undo_Lists is new Keyed_Lists
     (Transaction_Ref, Logging_Technique_Ref, Boolean);

   package Transaction_Lists is new Linked_Lists (Transaction_Ref);

   type Undo_Redoable_Object_Type is new Memory_Object_Type with record
      Factory : Logging_Factory_Ref;
      Undo_List : Undo_Lists.List_Type;
      Redo_List : Undo_Lists.List_Type;
      Trans_List : Transaction_Lists.List_Type;
   end record;

   procedure Real_Abort (Object : in out Undo_Redoable_Object_Type;
                         Transaction : in Transaction_Ref);

   procedure Real_Commit (Object : in out Undo_Redoable_Object_Type;
                          Transaction : in Transaction_Ref);

   procedure Finalize (Object : in out Undo_Redoable_Object_Type);

   function OK_To_Propagate (Object : Undo_Redoable_Object_Type)
      return Boolean;

end Memory_Objects.Concrete.Undo_Redoable;
