-------------------------------------------------------------------------------
--                   Memory Objects.Concrete.Versioned                       --
--                                                                           --
--  This type implements the versioned memory objects.                       --
--  It uses a stack of memory storage to store the different states of       --
--  a data object                                                            --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/08/18 11:48:41 $
--  $RCSfile: memory_objects-concrete-versioned.ads,v $
--  $Revision: 1.1 $
--                                                                           --
-------------------------------------------------------------------------------

with Cache_Managers; use Cache_Managers;
with Streams; use Streams;
with Storages.Volatile.Memory_Storages;
use Storages.Volatile.Memory_Storages;

package Memory_Objects.Concrete.Versioned is

   type Versioned_Object_Type is new Memory_Object_Type with private;
   type Versioned_Object_Ref is access all Versioned_Object_Type'Class;

   function Create (Cache : Cache_Manager_Ref) return Versioned_Object_Ref;

   procedure Execute (Object      : in out Versioned_Object_Type;
                      Operation   : in out Normal_Operation_Type'Class;
                      Transaction : in Transaction_Ref);

private

   type Node_Type;

   type Node_Ref is access Node_Type;

   type Node_Type is record
      Next        : Node_Ref;
      Stream      : Abstract_Stream_Ref;
      State       : Memory_Storage_Ref;
      Transaction : Transaction_Ref;
   end record;

   type Versioned_Object_Type is new Memory_Object_Type with record
      Versionlist : Node_Ref;
   end record;

   procedure Check_And_Then_Insert (Object      : in out Versioned_Object_Type;
                                    Transaction : in Transaction_Ref);

   procedure Real_Discard (Object : in out Versioned_Object_Type;
                           Transaction : in Transaction_Ref);

end Memory_Objects.Concrete.Versioned;
