with Crash_Recovery;
with Ada.Unchecked_Deallocation;

package body Persistent_Objects.Cataloged is

   procedure Free is new Ada.Unchecked_Deallocation
     (Cataloged_Object_Type'Class, Cataloged_Object_Ref);

   procedure Save_Object (Object : in out Cataloged_Object_Type) is
   begin
      if Object.Saved_Once = False then
         Crash_Recovery.Object_First_Save_Begin (Object.Params.all);
         Object.Saved_Once := True;
         Save_Object (Persistent_Object_Type (Object));
         Crash_Recovery.Object_First_Save_End (Object.Params.all);
      else
         Save_Object (Persistent_Object_Type (Object));
      end if;
   end Save_Object;

   procedure Initialize_Object
      (Object : in out Cataloged_Object_Ref;
       Storage_Params : in Non_Volatile_Params_Type'Class) is
   begin
      Crash_Recovery.Object_Creation_Begin (Storage_Params);
      Initialize_Object (Persistent_Object_Ref (Object), Storage_Params);
      Crash_Recovery.Object_Creation_End (Storage_Params);
   end Initialize_Object;

   procedure Delete_Object
     (Object : in out Cataloged_Object_Ref) is
      Storage_Params : Non_Volatile_Params_Type'Class :=
        Object.Params.all;
   begin
      Crash_Recovery.Object_Deletion_Begin (Object.Params.all);
      Open (Object.Storage.all, Out_Mode);
      Delete (Object.Storage.all);
      Free (Object);
      Crash_Recovery.Object_Deletion_End (Storage_Params);
   end Delete_Object;

end Persistent_Objects.Cataloged;
