
package Persistent_Objects.Directories is

   type Directory_Type is new Persistent_Object_Type with private;
   type Directory_Ref is access all Directory_Type'Class;

   function Create_Object (Storage_Params : Non_Volatile_Params_Type'Class)
      return Directory_Ref;

   procedure Deallocate_Object (Directory : in out Directory_Type);

   procedure Insert (Directory : in out Directory_Type;
                     Params    : in Non_Volatile_Params_Type'Class);

   procedure Validate (Directory : in out Directory_Type;
                       Params    : in Non_Volatile_Params_Type'Class);

   procedure Check_Existence (Directory : in Directory_Type;
                              Params    : in Non_Volatile_Params_Type'Class;
                              Valid     : out Boolean;
                              Exists    : out Boolean);

   procedure Delete (Directory : in out Directory_Type;
                     Params    : in Non_Volatile_Params_Type'Class);

   function Valid_Get (Directory : Directory_Type;
                       Number    : Natural)
                       return Non_Volatile_Params_Type'Class;

   function Real_Get (Directory : Directory_Type;
                      Number    : Natural)
                      return Non_Volatile_Params_Type'Class;

   function Number_Of_Real_Objects (Directory : in Directory_Type)
     return Natural;

   function Number_Of_Valid_Objects (Directory : in Directory_Type)
     return Natural;

   Params_Not_Found : exception;

private

   type Object_Params_Node_Type;
   type Object_Params_Node_Ref is access all Object_Params_Node_Type;

   type Object_Params_Node_Type is record
      Params : Non_Volatile_Params_Ref;
      Valid  : Boolean := False;
      Next   : Object_Params_Node_Ref := null;
   end record;

   type Directory_Type is new Persistent_Object_Type with record
      Number      : Natural := 0;
      Real_Number : Natural := 0;
      Params_List : Object_Params_Node_Ref := null;
   end record;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item   : Directory_Type);
   for Directory_Type'Write use My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item   : out Directory_Type);
   for Directory_Type'Read use My_Read;

end Persistent_Objects.Directories;
