
with Ada.Unchecked_Deallocation;

package body Persistent_Objects.Directory_Logs is

   procedure Free is new Ada.Unchecked_Deallocation
     (Non_Volatile_Params_Type'Class, Non_Volatile_Params_Ref);

   function Create_Object (Storage_Params : Non_Volatile_Params_Type'Class;
                           Action : Action_Type;
                           Object : Non_Volatile_Params_Type'Class)
                           return Directory_Log_Ref is
      Result : Directory_Log_Ref := new Directory_Log_Type;
   begin
      Result.Action := Action;
      Result.Object := new Non_Volatile_Params_Type'Class' (Object);
      Initialize_Object (Persistent_Object_Ref (Result), Storage_Params);
      Save_Object (Result.all);
      return Result;
   end Create_Object;

   procedure Delete_Object (Log : in out Directory_Log_Type) is
   begin
      Open (Log.Storage.all, Out_Mode);
      Delete (Log.Storage.all);
      if Log.Object /= null then
         Free (Log.Object);
      end if;
   end Delete_Object;

   function Exists (Storage_Params : Non_Volatile_Params_Type'Class)
                    return Boolean is
   begin
      return True;
   end Exists;

   function Get_Object_Params (Log : Directory_Log_Type) return
     Non_Volatile_Params_Type'Class is
   begin
      return Log.Object.all;
   end Get_Object_Params;

   function Get_Action (Log : Directory_Log_Type) return Action_Type is
   begin
      return Log.Action;
   end Get_Action;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item   : Directory_Log_Type) is
   begin
      Action_Type'Write (Stream, Item.Action);
      if Item.Object /= null then
         Non_Volatile_Params_Type'Class'Output (Stream, Item.Object.all);
      end if;
   end My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item   : out Directory_Log_Type) is
   begin
      Action_Type'Read (Stream, Item.Action);
      Item.Object := new Non_Volatile_Params_Type'Class'
        (Non_Volatile_Params_Type'Class'Input (Stream));
   end My_Read;

end Persistent_Objects.Directory_Logs;
