
with Storages.Non_Volatile; use Storages.Non_Volatile;
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;
with Ada.Finalization; use Ada.Finalization;
with Ada.Streams; use Ada.Streams;
with Streams; use Streams;

package Persistent_Objects is

   type Persistent_Object_Type is tagged private;
   type Persistent_Object_Ref is access all Persistent_Object_Type'Class;

   procedure Save_Object (Object : in out Persistent_Object_Type);

   function Restore_Object (Storage_Params : in Non_Volatile_Params_Type'Class)
      return Persistent_Object_Ref;

   --  Every derived class must implement a Create_Object function, that will
   --  allocate a new instance of the class, perform any initialization
   --  operations and then up-call Initialize_Object.

   procedure Initialize_Object
      (Object : in out Persistent_Object_Ref;
       Storage_Params : in Non_Volatile_Params_Type'Class);

   Object_Non_Existant,
     Object_Existant,
     Object_Corrupt : exception;

private

   type Persistent_Object_Type is new Controlled with record
      Storage : Non_Volatile_Storage_Ref := null;
      Params  : Non_Volatile_Params_Ref := null;
   end record;

   procedure Finalize (S : in out Persistent_Object_Type);

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item   : in Persistent_Object_Type);
   for Persistent_Object_Type'Write use My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item   : out Persistent_Object_Type);
   for Persistent_Object_Type'Read use My_Read;

end Persistent_Objects;



