-------------------------------------------------------------------------------
--                                 Atomic Calls                              --
--                                                                           --
--  This is the root type for atomic operations                              --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2001/04/23 11:19:11 $
--  $RCSfile: atomic_calls.adb,v $
--  $Revision: 1.4 $
--                                                                           --
-------------------------------------------------------------------------------

with Memory_Objects; use Memory_Objects;
with Transactions; use Transactions;
with Transaction_Identifiers.Task_Attribute_Interface;
use Transaction_Identifiers.Task_Attribute_Interface;
with Transactions; use Transactions;
with Lock_Managers; use Lock_Managers;
with Recovery_Support; use Recovery_Support;
with Recovery_Managers; use Recovery_Managers;

package body Atomic_Calls is

   procedure Atomic_Do (Call : in Atomic_Call_Type;
                        Operation : in out Normal_Operation_Type'Class) is
      My_Lock_Manager : Lock_Manager_Ref :=
        Get_Lock_Manager (Call.My_Memory_Object.all);
      My_Recovery_Manager : Recovery_Manager_Ref := Get_Recovery_Manager;
      Current_Transaction : Transaction_Ref := Get_Current_Transaction;
   begin
      if Current_Transaction = null then
         raise No_Current_Transaction;
      end if;
      Request_Lock_And_Mutex (My_Lock_Manager.all, Operation,
                              Current_Transaction);
      Pre_Operation (My_Recovery_Manager.all, Call.My_Memory_Object,
                     Current_Transaction);

      if Is_Update (Operation) then
         Check_And_Then_Add_Modified
           (Current_Transaction.all,
            Memory_Object_Ref (Call.My_Memory_Object));
      else
         Check_And_Then_Add
           (Current_Transaction.all,
            Memory_Object_Ref (Call.My_Memory_Object));
      end if;

      begin
         Execute (Call.My_Memory_Object.all, Operation, Current_Transaction);
         Post_Operation (My_Recovery_Manager.all, Call.My_Memory_Object,
                         Current_Transaction);
         Free_Mutex (My_Lock_Manager.all);
      exception
         when others =>
            Post_Operation (My_Recovery_Manager.all, Call.My_Memory_Object,
                            Current_Transaction);
            Free_Mutex (My_Lock_Manager.all);
            raise;
      end;
   end Atomic_Do;

end Atomic_Calls;
