-------------------------------------------------------------------------------
--                      Log Information.Object.Create                        --
--                                                                           --
--  This is the type that is used to write Create information to the log     --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/10/02 13:08:41 $
--  $RCSfile: log_information-object-create.adb,v $
--  $Revision: 1.1 $
--                                                                           --
-------------------------------------------------------------------------------

with Streams; use Streams;
with Ada.IO_Exceptions; use Ada.IO_Exceptions;

with Memory_Objects.Concrete; use Memory_Objects.Concrete;

with Storages; use Storages;
with Storages.Non_Volatile; use Storages.Non_Volatile;
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;

with Ada.Unchecked_Deallocation;

package body Log_Information.Object.Create is

   function Create (Params : in Non_Volatile_Params_Type'Class) return
     Create_Information_Type is
      Result : Create_Information_Type;
   begin
      Result.Params := new Non_Volatile_Params_Type'Class' (Params);
      return Result;
   end Create;

   procedure Free is new Ada.Unchecked_Deallocation
     (Stream_Type'Class, Stream_Ref);

   procedure Free is new Ada.Unchecked_Deallocation
     (Non_Volatile_Storage_Type'Class, Non_Volatile_Storage_Ref);

   procedure Delete (Creation : in Create_Information_Type) is
      Store : Non_Volatile_Storage_Ref := Open_Storage
        (Creation.Params.all, In_Mode);
      Temp_Stream : Stream_Ref := new Stream_Type (Store);
      State_Identifier : Natural;
      To_Delete : Boolean := False;
   begin
      begin
         Natural'Read (Temp_Stream, State_Identifier);
         if State_Identifier = Initial_State then
            To_Delete := True;
         end if;
      exception
         when Data_Not_Available | End_Error =>
            To_Delete := True;
      end;
      Close (Store.all);
      Free (Temp_Stream);
      if To_Delete then
         Open (Store.all, Out_Mode);
         Delete (Store.all);
      end if;
      Free (Store);
   exception
      when Storage_Not_Existant =>
         --  we crashed before really creating the object
         --  everything's ok
         null;
   end Delete;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item : Create_Information_Type) is
   begin
      Simple_Transaction_Identifier_Type'Write (Stream, Item.Transaction);
      Non_Volatile_Params_Type'Class'Output (Stream, Item.Params.all);
   end My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item : out Create_Information_Type) is
   begin
      Simple_Transaction_Identifier_Type'Read (Stream, Item.Transaction);
      Item.Params := new Non_Volatile_Params_Type'Class'
        (Non_Volatile_Params_Type'Class'Input (Stream));
   end My_Read;

end Log_Information.Object.Create;
