-------------------------------------------------------------------------------
--                      Log Information.Object.Redo                         --
--                                                                           --
--  This is the type that is used to write Redo information to the log       --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/10/02 13:08:43 $
--  $RCSfile: log_information-object-redo.adb,v $
--  $Revision: 1.1 $
--                                                                           --
-------------------------------------------------------------------------------

with Logging_Techniques; use Logging_Techniques;
with Memory_Objects.Concrete; use Memory_Objects.Concrete;

package body Log_Information.Object.Redo is

   function Create (Object : in Memory_Object_Ref;
                    Redo_List : Logging_Technique_Array_Type) return
     Redo_Information_Type is
   begin
      if Redo_List'Length = 0 then
         raise Constraint_Error;
      end if;

      declare
         Result : Redo_Information_Type (Redo_List'Length);
      begin
         Result.Params := new Non_Volatile_Params_Type'Class'
           (Get_Params (Concrete_Memory_Object_Ref (Object) .all));
         Result.State_Identifier := Get_State_Identifier
           (Concrete_Memory_Object_Ref (Object) .all);
         Result.Information := Redo_List;
         return Result;
      end;
   end Create;

   procedure Redo (Info : in Redo_Information_Type;
                   Object : in Memory_Object_Ref;
                   Loading : in Loading_Operation_Type'Class) is
   begin
      for I in reverse Info.Information'Range loop
         Logging_Techniques.Redo (Info.Information (I).all, Object);
      end loop;
   end Redo;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item : Redo_Information_Type) is
   begin
      Simple_Transaction_Identifier_Type'Write (Stream, Item.Transaction);
      Non_Volatile_Params_Type'Class'Output (Stream, Item.Params.all);
      Natural'Write (Stream, Item.State_Identifier);
      for I in Item.Information'Range loop
         Logging_Technique_Type'Class'Output
           (Stream, Item.Information (I).all);
      end loop;
   end My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item : out Redo_Information_Type) is
   begin
      Simple_Transaction_Identifier_Type'Read (Stream, Item.Transaction);
      Item.Params := new Non_Volatile_Params_Type'Class'
        (Non_Volatile_Params_Type'Class'Input (Stream));
      Natural'Read (Stream, Item.State_Identifier);
      for I in Item.Information'Range loop
         Item.Information (I) := new Logging_Technique_Type'Class'
            (Logging_Technique_Type'Class'Input (Stream));
      end loop;
   end My_Read;

end Log_Information.Object.Redo;
