-------------------------------------------------------------------------------
--                    Log Information.Object.Undo                            --
--                                                                           --
--  This is the type that is used to write undo information to the log       --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/10/02 13:08:44 $
--  $RCSfile: log_information-object-undo.adb,v $
--  $Revision: 1.1 $
--                                                                           --
-------------------------------------------------------------------------------

with Memory_Objects.Concrete; use Memory_Objects.Concrete;

package body Log_Information.Object.Undo is

   function Create (Object : in Memory_Object_Ref;
                    Undo_List : Logging_Technique_Array_Type) return
     Undo_Information_Type is
   begin
      if Undo_List'Length = 0 then
         raise Constraint_Error;
      end if;

      declare
         Result : Undo_Information_Type (Undo_List'Length);
      begin
         Result.Params := new Non_Volatile_Params_Type'Class'
           (Get_Params (Concrete_Memory_Object_Ref (Object) .all));
         Result.State_Identifier := Get_State_Identifier
           (Concrete_Memory_Object_Ref (Object) .all);
         Result.Information := Undo_List;
         return Result;
      end;
   end Create;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item : Undo_Information_Type) is
   begin
      Simple_Transaction_Identifier_Type'Write (Stream, Item.Transaction);
      Non_Volatile_Params_Type'Class'Output (Stream, Item.Params.all);
      Natural'Write (Stream, Item.State_Identifier);
      for I in Item.Information'Range loop
         Logging_Technique_Type'Class'Output
           (Stream, Item.Information (I).all);
      end loop;
   end My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item : out Undo_Information_Type) is
   begin
      Simple_Transaction_Identifier_Type'Read (Stream, Item.Transaction);
      Item.Params := new Non_Volatile_Params_Type'Class'
        (Non_Volatile_Params_Type'Class'Input (Stream));
      Natural'Read (Stream, Item.State_Identifier);
      for I in Item.Information'Range loop
         Item.Information (I) := new Logging_Technique_Type'Class'
            (Logging_Technique_Type'Class'Input (Stream));
      end loop;
   end My_Read;

end Log_Information.Object.Undo;
