-------------------------------------------------------------------------------
--                       Logging Techniques.Logical                          --
--                                                                           --
--  This is the type that encapsulates logical logging                       --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/09/29 07:18:17 $
--  $RCSfile: logging_techniques-logical.adb,v $
--  $Revision: 1.4 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Unchecked_Deallocation;
with Memory_Objects.Concrete; use Memory_Objects.Concrete;

package body Logging_Techniques.Logical is

   procedure Free_Operation is new Ada.Unchecked_Deallocation
     (Normal_Operation_Type'Class, Normal_Operation_Ref);

   function Create (Operation : Normal_Operation_Type'Class)
                    return Logical_Logging_Ref is
      Result : Logical_Logging_Ref := new Logical_Logging_Type;
   begin
      Result.Operation := new Normal_Operation_Type'Class' (Operation);
      return Result;
   end Create;

   function Create (Factory : Logical_Factory_Type;
                    Operation : Normal_Operation_Type'Class;
                    Saving_Operation : Saving_Operation_Ref;
                    Data_Object : Data_Ref)
     return Logging_Technique_Ref is
   begin
      return Logging_Technique_Ref (Create (Operation));
   end Create;

   function Is_Physical (Factory : Logical_Factory_Type) return Boolean is
   begin
      return False;
   end Is_Physical;

   procedure Undo (Technique : in Logical_Logging_Type;
                   Object : in Memory_Object_Ref) is
   begin
      Execute_Undo (Concrete_Memory_Object_Ref (Object) .all,
                    Technique.Operation.all, null);
   end Undo;

   procedure Redo (Technique : in Logical_Logging_Type;
                   Object : in Memory_Object_Ref) is
   begin
      Execute (Memory_Object_Type (Object.all),
               Technique.Operation.all, null);
   end Redo;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item : in Logical_Logging_Type) is
   begin
      Normal_Operation_Type'Class'Output (Stream, Item.Operation.all);
   end My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item : out Logical_Logging_Type) is
   begin
      Item.Operation := new Normal_Operation_Type'Class'
        (Normal_Operation_Type'Class'Input (Stream));
   end My_Read;

   procedure Finalize (Logical : in out Logical_Logging_Type) is
   begin
      --  Free_Operation (Logical.Operation);
      null;
   end Finalize;

end Logging_Techniques.Logical;
