-------------------------------------------------------------------------------
--                       Logging Techniques.Logical                          --
--                                                                           --
--  This is the type that encapsulates logical logging                       --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/09/29 07:18:18 $
--  $RCSfile: logging_techniques-logical.ads,v $
--  $Revision: 1.4 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Streams; use Ada.Streams;

with Memory_Objects; use Memory_Objects;
with Operations.Normal; use Operations.Normal;

package Logging_Techniques.Logical is

   type Logical_Logging_Type is new Logging_Technique_Type with private;
   type Logical_Logging_Ref is access all Logical_Logging_Type;

   function Create (Operation : Normal_Operation_Type'Class)
      return Logical_Logging_Ref;

   procedure Undo (Technique : in Logical_Logging_Type;
                   Object : in Memory_Object_Ref);

   procedure Redo (Technique : in Logical_Logging_Type;
                   Object : in Memory_Object_Ref);

   type Logical_Factory_Type is new Logging_Factory_Type with null record;
   type Logical_Factory_Ref is access all Logical_Factory_Type'Class;

   function Create (Factory : Logical_Factory_Type;
                    Operation : Normal_Operation_Type'Class;
                    Saving_Operation : Saving_Operation_Ref;
                    Data_Object : Data_Ref)
     return Logging_Technique_Ref;

   function Is_Physical (Factory : Logical_Factory_Type) return Boolean;

private

   type Logical_Logging_Type is new Logging_Technique_Type with record
      Operation : Normal_Operation_Ref;
   end record;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item : in Logical_Logging_Type);
   for Logical_Logging_Type'Write use My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item : out Logical_Logging_Type);
   for Logical_Logging_Type'Read use My_Read;

   procedure Finalize (Logical : in out Logical_Logging_Type);

end Logging_Techniques.Logical;
