-------------------------------------------------------------------------------
--                       Logging Techniques.Physical                         --
--                                                                           --
--  This is the type that encapsulates physical logging                      --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/09/29 07:18:18 $
--  $RCSfile: logging_techniques-physical.adb,v $
--  $Revision: 1.4 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Unchecked_Deallocation;
with Ada.Streams; use Ada.Streams;

with Memory_Objects.Concrete; use Memory_Objects.Concrete;

package body Logging_Techniques.Physical is

   procedure Free_State is new Ada.Unchecked_Deallocation
     (Memory_Storage_Type'Class, Memory_Storage_Ref);

   procedure Free_Stream is new Ada.Unchecked_Deallocation
     (Stream_Type'Class, Stream_Ref);

   function Create (Saving_Operation : Saving_Operation_Ref;
                    Data_Object : Data_Ref) return Physical_Logging_Ref is
      Result : Physical_Logging_Ref := new Physical_Logging_Type;
   begin
      Result.State := new Memory_Storage_Type;
      Result.Stream := new Stream_Type (Result.State);
      Save (Saving_Operation.all, Data_Object,
            Abstract_Stream_Ref (Result.Stream));
      return Result;
   end Create;

   function Create (Factory : Physical_Factory_Type;
                    Operation : Normal_Operation_Type'Class;
                    Saving_Operation : Saving_Operation_Ref;
                    Data_Object : Data_Ref)
     return Logging_Technique_Ref is
   begin
      return Logging_Technique_Ref (Create (Saving_Operation, Data_Object));
   end Create;

   function Is_Physical (Factory : Physical_Factory_Type) return Boolean is
   begin
      return True;
   end Is_Physical;

   procedure Undo (Technique : in Physical_Logging_Type;
                   Object : in Memory_Object_Ref) is
   begin
      Execute_Load (Concrete_Memory_Object_Ref (Object) .all,
                    Abstract_Stream_Ref (Technique.Stream));
   end Undo;

   procedure Redo (Technique : in Physical_Logging_Type;
                   Object : in Memory_Object_Ref) is
   begin
      Execute_Load (Concrete_Memory_Object_Ref (Object) .all,
                    Abstract_Stream_Ref (Technique.Stream));
   end Redo;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item : in Physical_Logging_Type) is
   begin
      Stream_Element_Array'Output
        (Stream, To_Stream_Element_Array (Item.State.all));
   end My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item : out Physical_Logging_Type) is
      Result : Physical_Logging_Type;
   begin
      Result.State := new Memory_Storage_Type;
      Result.Stream := new Stream_Type (Result.State);
      declare
         Tmp : Stream_Element_Array := Stream_Element_Array'Input (Stream);
      begin
         Stream_Element_Array'Write (Result.Stream, Tmp);
      end;
      Item := Result;
   end My_Read;

   procedure Finalize (Physical : in out Physical_Logging_Type) is
   begin -- Finalize
      null;
      --  Free_State (Physical.State);
      --  Free_Stream (Physical.Stream);
   end Finalize;

end Logging_Techniques.Physical;
