-------------------------------------------------------------------------------
--                       Logging Techniques.Physical                         --
--                                                                           --
--  This is the type that encapsulates physical logging                      --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/09/29 07:18:19 $
--  $RCSfile: logging_techniques-physical.ads,v $
--  $Revision: 1.4 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Streams;
with Streams; use Streams;
with Storages.Volatile.Memory_Storages; use Storages.Volatile.Memory_Storages;
with Operations.Saving; use Operations.Saving;
with Memory_Objects; use Memory_Objects;

package Logging_Techniques.Physical is

   type Physical_Logging_Type is new Logging_Technique_Type with private;
   type Physical_Logging_Ref is access all Physical_Logging_Type;

   function Create (Saving_Operation : Saving_Operation_Ref;
                    Data_Object : Data_Ref)
      return Physical_Logging_Ref;

   procedure Undo (Technique : in Physical_Logging_Type;
                   Object : in Memory_Object_Ref);

   procedure Redo (Technique : in Physical_Logging_Type;
                   Object : in Memory_Object_Ref);

   type Physical_Factory_Type is new Logging_Factory_Type with null record;
   type Physical_Factory_Ref is access all Physical_Factory_Type'Class;

   function Create (Factory : Physical_Factory_Type;
                    Operation : Normal_Operation_Type'Class;
                    Saving_Operation : Saving_Operation_Ref;
                    Data_Object : Data_Ref)
     return Logging_Technique_Ref;

   function Is_Physical (Factory : Physical_Factory_Type) return Boolean;

private

   type Physical_Logging_Type is new Logging_Technique_Type with record
      State : Memory_Storage_Ref;
      Stream : Stream_Ref;
   end record;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item : in Physical_Logging_Type);
   for Physical_Logging_Type'Write use My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item : out Physical_Logging_Type);
   for Physical_Logging_Type'Read use My_Read;

   procedure Finalize (Physical : in out Physical_Logging_Type);

end Logging_Techniques.Physical;
