-------------------------------------------------------------------------------
--                           Logging Techniques                              --
--                                                                           --
--  This is the abstract root type for the logging techniques                --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/09/29 07:18:21 $
--  $RCSfile: logging_techniques.ads,v $
--  $Revision: 1.5 $
--                                                                           --
-------------------------------------------------------------------------------

with Memory_Objects; use Memory_Objects;
with Operations.Saving; use Operations.Saving;
with Operations.Normal; use Operations.Normal;

with Ada.Finalization; use Ada.Finalization;

package Logging_Techniques is

   type Logging_Technique_Type is abstract tagged private;
   type Logging_Technique_Ref is access all Logging_Technique_Type'Class;

   procedure Undo (Technique : in Logging_Technique_Type;
                   Object : in Memory_Object_Ref) is abstract;

   procedure Redo (Technique : in Logging_Technique_Type;
                   Object : in Memory_Object_Ref) is abstract;

   type Logging_Technique_Array_Type is array (Natural range <>) of
     Logging_Technique_Ref;

   type Logging_Factory_Type is abstract tagged null record;
   type Logging_Factory_Ref is access all Logging_Factory_Type'Class;

   function Create (Factory : Logging_Factory_Type;
                    Operation : Normal_Operation_Type'Class;
                    Saving_Operation : Saving_Operation_Ref;
                    Data_Object : Data_Ref)
     return Logging_Technique_Ref is abstract;

   function Is_Physical (Factory : Logging_Factory_Type) return Boolean
     is abstract;

private

   type Logging_Technique_Type is abstract new Controlled
     with null record;

end Logging_Techniques;
