-------------------------------------------------------------------------------
--                                    Logs                                   --
--                                                                           --
--  This package declares the log type and its operations                    --
--                                                                           --
--  $Author: xcaron $
--  $Date: 2000/11/30 12:25:24 $
--  $RCSfile: logs.ads,v $
--  $Revision: 1.5 $
--                                                                           --
-------------------------------------------------------------------------------

with Transactions; use Transactions;

with Storages.Non_Volatile.Stable; use Storages.Non_Volatile.Stable;
with Storage_Params.Non_Volatile;
use Storage_Params.Non_Volatile;
with Storage_Params.Non_Volatile.Stable;
use Storage_Params.Non_Volatile.Stable;

with Memory_Objects; use Memory_Objects;

with Log_Information; use Log_Information;

with Streams; use Streams;
with Buffers.Unbounded; use Buffers.Unbounded;

package Logs is

   type Log_Type is tagged limited private;
   type Log_Ref is access all Log_Type'Class;

   procedure Initialize (Log : in out Log_Type;
                         Recovery_Needed : out Boolean;
                         Anchor : in Stable_Params_Type'Class;
                         Params : in Stable_Params_Type'Class);

   procedure Log_Transaction_Status (Log : in out Log_Type;
                                     Transaction : in Transaction_Ref;
                                     Status : in Transaction_Status_Type);

   procedure Log_Undo_Information (Log : in out Log_Type;
                                   Transaction : in Transaction_Ref;
                                   Object : in Memory_Object_Ref);

   procedure Log_Redo_Information (Log : in out Log_Type;
                                   Transaction : in Transaction_Ref;
                                   Object : in Memory_Object_Ref);

   procedure Log_Creation_Information
     (Log : in out Log_Type;
      Transaction : in Transaction_Ref;
      Params : in Non_Volatile_Params_Type'Class);

   procedure Start_Recovery (Log : in out Log_Type);

   procedure End_Recovery (Log : in out Log_Type);

   function Get_Next_Log_Information (Log : in Log_Type)
      return Log_Information_Type'Class;

   procedure Shutdown (Log : in out Log_Type);

   Log_Corrupt, No_More_Info : exception;

private

   type System_Status_Type is
      (Normal, Recovery_Needed);

   type Anchor_Type is record
      Status : System_Status_Type;
   end record;

   protected type Log_Protection_Type is
      procedure Log_Transaction_Status (Log : in out Log_Type;
                                        Transaction : in Transaction_Ref;
                                        Status : in Transaction_Status_Type);

      procedure Log_Undo_Information (Log : in out Log_Type;
                                      Transaction : in Transaction_Ref;
                                      Object : in Memory_Object_Ref);

      procedure Log_Redo_Information (Log : in out Log_Type;
                                      Transaction : in Transaction_Ref;
                                      Object : in Memory_Object_Ref);

      procedure Log_Creation_Information
        (Log : in out Log_Type;
         Transaction : in Transaction_Ref;
         Params : in Non_Volatile_Params_Type'Class);

   end Log_Protection_Type;

   type Log_Type is tagged limited record
      Params          : Stable_Params_Ref := null;
      Log_Storage     : Stable_Storage_Ref := null;
      Log_Stream      : Buffered_Stream_Ref := null;
      Log_Buffer      : Unbounded_Buffer_Ref := null;
      Anchor_Storage  : Stable_Storage_Ref := null;
      Anchor          : Anchor_Type;
      Protection      : Log_Protection_Type;
   end record;

end Logs;
