-------------------------------------------------------------------------------
--                            Redo Recovery Manager                          --
--                                                                           --
--  This is the type for the redo recovery manager                           --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/09/29 07:18:22 $
--  $RCSfile: recovery_managers-redo.ads,v $
--  $Revision: 1.4 $
--                                                                           --
-------------------------------------------------------------------------------

with Storage_Params.Non_Volatile;
use Storage_Params.Non_Volatile;
with Storage_Params.Non_Volatile.Stable;
use Storage_Params.Non_Volatile.Stable;

package Recovery_Managers.Redo is

   type Redo_Recovery_Manager_Type is new Recovery_Manager_Type with private;

   type Redo_Recovery_Manager_Ref is access all
      Redo_Recovery_Manager_Type'Class;

   function Create (Log_Anchor  : in Stable_Params_Type'Class;
                    Log_Storage : in Stable_Params_Type'Class)
     return Redo_Recovery_Manager_Ref;

   procedure Pre_Operation (Manager : in out Redo_Recovery_Manager_Type;
                            Object  : in Concrete_Memory_Object_Ref;
                            Transaction : in Transaction_Ref);

   procedure Post_Operation (Manager : in out Redo_Recovery_Manager_Type;
                             Object  : in Concrete_Memory_Object_Ref;
                             Transaction : in Transaction_Ref);

   procedure Recover (Manager : in out Redo_Recovery_Manager_Type);

   procedure Recover_Object (Manager : in out Redo_Recovery_Manager_Type;
                             Object : in Concrete_Memory_Object_Ref;
                             Params : in Non_Volatile_Params_Ref;
                             Loading : in Loading_Operation_Type'Class);

   procedure Abort_Transaction (Manager : in out Redo_Recovery_Manager_Type;
                                Transaction : in Transaction_Ref);

   procedure Commit_Transaction (Manager : in out Redo_Recovery_Manager_Type;
                                 Transaction : in Transaction_Ref);

private

   type Redo_Recovery_Manager_Type is new Recovery_Manager_Type with
     null record;

end Recovery_Managers.Redo;
