-------------------------------------------------------------------------------
--                              Recovery Manager                             --
--                                                                           --
--  This is the abstract root type for the recovery manager                  --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/12/13 15:54:11 $
--  $RCSfile: recovery_managers.adb,v $
--  $Revision: 1.9 $
--                                                                           --
-------------------------------------------------------------------------------

with Memory_Objects; use Memory_Objects;
with Recovery_Support; use Recovery_Support;
with Cache_Managers; use Cache_Managers;

package body Recovery_Managers is

   procedure Begin_Transaction (Manager : in out Recovery_Manager_Type;
                                Transaction : in Transaction_Ref) is
   begin
      Log_Transaction_Status (Manager.Log, Transaction, Open);
   end Begin_Transaction;

   procedure Create (Manager  : in out Recovery_Manager_Type;
                     Object   : out Memory_Object_Ref;
                     Transaction : in Transaction_Ref;
                     Params   : Non_Volatile_Params_Type'Class;
                     Creation : Creation_Operation_Type'Class;
                     Loading  : Loading_Operation_Type'Class;
                     Saving   : Saving_Operation_Type'Class;
                     Deletion : Deletion_Operation_Type'Class) is
   begin
      Log_Creation_Information (Manager.Log, Transaction, Params);
      Object := Create (Get_Cache_Manager.all, Params, Creation,
                        Loading, Saving, Deletion);
   end Create;

   procedure Initialize (Manager : in out Recovery_Manager_Type;
                         Log_Anchor : in Stable_Params_Type'Class;
                         Log_Storage : in Stable_Params_Type'Class) is
      Recovery_Needed : Boolean;
   begin
      Initialize (Manager.Log, Recovery_Needed, Log_Anchor, Log_Storage);
      if Recovery_Needed then
         Recover (Recovery_Manager_Type'Class (Manager));
      end if;
   end Initialize;

   procedure Shutdown (Manager : in out Recovery_Manager_Type) is
   begin
      Shutdown (Manager.Log);
   end Shutdown;

end Recovery_Managers;
