-------------------------------------------------------------------------------
--                              Recovery Manager                             --
--                                                                           --
--  This is the abstract root type for the recovery manager                  --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/10/02 13:08:58 $
--  $RCSfile: recovery_managers.ads,v $
--  $Revision: 1.8 $
--                                                                           --
-------------------------------------------------------------------------------

with Transactions; use Transactions;
with Memory_Objects; use Memory_Objects;
with Memory_Objects.Concrete; use Memory_Objects.Concrete;
with Logs; use Logs;

with Storage_Params.Non_Volatile;
use Storage_Params.Non_Volatile;
with Storage_Params.Non_Volatile.Stable;
use Storage_Params.Non_Volatile.Stable;
with Operations.Creation; use Operations.Creation;
with Operations.Loading; use Operations.Loading;
with Operations.Saving; use Operations.Saving;
with Operations.Deletion; use Operations.Deletion;


package Recovery_Managers is

   type Recovery_Manager_Type is abstract tagged limited private;

   type Recovery_Manager_Ref is access all Recovery_Manager_Type'Class;

   procedure Pre_Operation (Manager : in out Recovery_Manager_Type;
                            Object  : in Concrete_Memory_Object_Ref;
                            Transaction : in Transaction_Ref) is abstract;

   procedure Post_Operation (Manager : in out Recovery_Manager_Type;
                             Object  : in Concrete_Memory_Object_Ref;
                             Transaction : in Transaction_Ref) is abstract;

   procedure Recover (Manager : in out Recovery_Manager_Type) is abstract;

   procedure Recover_Object (Manager : in out Recovery_Manager_Type;
                             Object : in Concrete_Memory_Object_Ref;
                             Params : in Non_Volatile_Params_Ref;
                             Loading : in Loading_Operation_Type'Class)
      is abstract;

   procedure Begin_Transaction (Manager : in out Recovery_Manager_Type;
                                Transaction : in Transaction_Ref);

   procedure Abort_Transaction (Manager : in out Recovery_Manager_Type;
                                Transaction : in Transaction_Ref)
      is abstract;

   procedure Commit_Transaction (Manager : in out Recovery_Manager_Type;
                                 Transaction : in Transaction_Ref)
     is abstract;

   procedure Create (Manager  : in out Recovery_Manager_Type;
                     Object   : out Memory_Object_Ref;
                     Transaction : in Transaction_Ref;
                     Params   : Non_Volatile_Params_Type'Class;
                     Creation : Creation_Operation_Type'Class;
                     Loading  : Loading_Operation_Type'Class;
                     Saving   : Saving_Operation_Type'Class;
                     Deletion : Deletion_Operation_Type'Class);

   procedure Shutdown (Manager : in out Recovery_Manager_Type);

private

   type Recovery_Manager_Type is abstract tagged limited record
      Log : Log_Type;
   end record;

   procedure Initialize (Manager : in out Recovery_Manager_Type;
                         Log_Anchor : in Stable_Params_Type'Class;
                         Log_Storage : in Stable_Params_Type'Class);

end Recovery_Managers;
