-------------------------------------------------------------------------------
--                                 Recovery_Support                          --
--                                                                           --
--  This package contains all operations necessary for starting-up and       --
--  configuring the recovery support                                         --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/08/31 14:30:39 $
--  $RCSfile: recovery_support.adb,v $
--  $Revision: 1.4 $
--                                                                           --
-------------------------------------------------------------------------------

with Cache_Managers.LRU; use Cache_Managers.LRU;
--  with Recovery_Managers.Undo; use Recovery_Managers.Undo;
with Recovery_Managers.Redo; use Recovery_Managers.Redo;
with Storage_Params.Non_Volatile.Stable.Mirrored_Storage_Params;
use Storage_Params.Non_Volatile.Stable.Mirrored_Storage_Params;

package body Recovery_Support is

   My_Cache_Manager : aliased LRU_Cache_Manager_Type;

   My_Recovery_Manager : Redo_Recovery_Manager_Ref;

   procedure System_Init is
      Anchor_Storage : Mirrored_Storage_Params_Type :=
        String_To_Storage_Params ("log_anchor");
      Log_Storage : Mirrored_Storage_Params_Type :=
        String_To_Storage_Params ("log_storage");
   begin
      System_Init (Anchor_Storage, Log_Storage);
   end System_Init;

   procedure System_Init (Anchor : in Stable_Params_Type'Class;
                          Params : in Stable_Params_Type'Class) is
   begin
      My_Recovery_Manager := Create (Anchor, Params);
   end System_Init;

   procedure System_Shutdown is
   begin
      Shutdown (My_Recovery_Manager.all);
   end System_Shutdown;

   function Get_Cache_Manager return Cache_Manager_Ref is
   begin
      return My_Cache_Manager'Access;
   end Get_Cache_Manager;

   function Get_Recovery_Manager return Recovery_Manager_Ref is
   begin
      return Recovery_Manager_Ref (My_Recovery_Manager);
   end Get_Recovery_Manager;

end Recovery_Support;


