-------------------------------------------------------------------------------
--                          Network Storage Params                           --
--                                                                           --
--  This package defines the network storage params type.                    --
--  It contains a reference to a universal storage parameter and defines     --
--  read and write procedures for it.                                        --
--  Watch out not to copy instances around, for only the reference to the    --
--  actual storage parameters will be copied!                                --
--                                                                           --
--  $Author: xcaron $
--  $Date: 2000/01/24 09:56:20 $
--  $RCSfile: network_storage_params.ads,v $
--  $Revision: 1.1 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Streams; use Ada.Streams;
with Universal_Storage_Params; use Universal_Storage_Params;

package Network_Storage_Params is

   pragma Remote_Types;

   type Network_Storage_Params_Type is private;

   function Create_Network_Params (P : in Universal_Storage_Params_Type'Class)
     return Network_Storage_Params_Type;

   function Extract_Params (P : Network_Storage_Params_Type)
     return Universal_Storage_Params_Type'Class;

private

   type Universal_Storage_Params_Ref is
      access all Universal_Storage_Params_Type'Class;

   type Network_Storage_Params_Type is record
      Params : Universal_Storage_Params_Ref;
   end record;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item   : in Network_Storage_Params_Type);
   for Network_Storage_Params_Type'Write use My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item   : out Network_Storage_Params_Type);
   for Network_Storage_Params_Type'Read use My_Read;

end Network_Storage_Params;
