-------------------------------------------------------------------------------
--                            Mirrored Storage Params                        --
--                                                                           --
--  Author: Xavier Caron                                                     --
--                                                                           --
--  This package provides the storage parameters for the mirrored            --
--  storage type.                                                            --
--                                                                           --
--  $Author: xcaron $
--  $Date: 2000/01/13 15:56:18 $
--  $Revision: 1.2 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Finalization; use Ada.Finalization;
with Ada.Streams; use Ada.Streams;

package Storage_Params.Non_Volatile.Stable.Mirrored_Storage_Params is

   type Mirrored_Storage_Params_Type is new Stable_Params_Type with private;

   --  By default, the function String_To_Storage_Params allocates the
   --  log and the two data locations, as local files with
   --  parameters as Mirrored_File_Params.
   function String_To_Storage_Params (S : in String)
     return Mirrored_Storage_Params_Type;

   --  The overloaded function String_To_Storage_Params allows
   --  the user to declare which non volatile storage params he wants
   --  to use for the log and the two data locations.
   function  String_To_Storage_Params (S : in String;
      Data_1_Params, Data_2_Params,
        Log_Params : in Non_Volatile_Params_Type'Class)
     return Mirrored_Storage_Params_Type;

   function Storage_Params_To_String (Params : in Mirrored_Storage_Params_Type)
     return String;

   function Get_Data_1_Params (Params : in Mirrored_Storage_Params_Type)
      return Non_Volatile_Params_Type'Class;

   function Get_Data_2_Params (Params : in Mirrored_Storage_Params_Type)
      return Non_Volatile_Params_Type'Class;

   function Get_Log_Params (Params : in Mirrored_Storage_Params_Type)
      return Non_Volatile_Params_Type'Class;

   function Create_Storage (Params : in Mirrored_Storage_Params_Type)
     return Storage_Ref;

   function Open_Storage (Params : in Mirrored_Storage_Params_Type;
                          Mode   : in Mode_Type := In_Mode)
     return Non_Volatile_Storage_Ref;

private

   subtype Index_Type is Natural range 0 .. 255;

   type Mirrored_Storage_Name_Type (N : Index_Type := 0) is record
      Name : String (1 .. N);
   end record;

   type Mirrored_Params_Members_Type is new Controlled with record
      Data_1_Params : Non_Volatile_Params_Ref;
      Data_2_Params : Non_Volatile_Params_Ref;
      Log_Params : Non_Volatile_Params_Ref;
   end record;

   procedure My_Write (Stream : access Ada.Streams.Root_Stream_Type'Class;
                       Item   : in Mirrored_Params_Members_Type);
   for Mirrored_Params_Members_Type'Write use My_Write;

   procedure My_Read (Stream : access Ada.Streams.Root_Stream_Type'Class;
                      Item   : out Mirrored_Params_Members_Type);
   for Mirrored_Params_Members_Type'Read use My_Read;

   procedure Finalize (P : in out Mirrored_Params_Members_Type);

   procedure Adjust (P : in out Mirrored_Params_Members_Type);

   type Mirrored_Storage_Params_Type is new Stable_Params_Type with
      record
         Mirrored_Storage_Name : Mirrored_Storage_Name_Type;
         Members : Mirrored_Params_Members_Type;
      end record;

end Storage_Params.Non_Volatile.Stable.Mirrored_Storage_Params;
