-------------------------------------------------------------------------------
--                                  Storages                                 --
--                                                                           --
--  This is the root type for any storage, volatile or non-volatile.         --
--  It is privately derived from Limited_Controlled, in order to be able     --
--  to perform clean-up work during finalization.                            --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/01/05 12:57:08 $
--  $RCSfile: storages.ads,v $
--  $Revision: 1.1.1.1 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Streams; use Ada.Streams;
with Ada.Finalization; use Ada.Finalization;

package Storages is

   type Storage_Type is abstract tagged limited private;
   type Storage_Ref is access all Storage_Type'Class;

   procedure Read (Storage : in out Storage_Type;
                   Item    : out Stream_Element_Array;
                   Last    : out Stream_Element_Offset) is abstract;
   --  Read a data item from the storage

   procedure Write (Storage : in out Storage_Type;
                    Item    : in Stream_Element_Array) is abstract;
   --  Append 'Data' to the storage

   function Get_Current_Size (Storage : in Storage_Type)
      return Stream_Element_Count is abstract;
   --  Returns the current size of the storage

   Data_Not_Available : exception;

private

   type Storage_Type is abstract new Limited_Controlled with null record;

end Storages;
