-------------------------------------------------------------------------------
--                             Unbounded Buffers                             --
--                                                                           --
--  The unbounded buffer uses a memory storage to store the data of calls    --
--  to Write. Only when Flush is called, the data is written to the          --
--  associated non-volatile storage.                                         --
--  On the first read, the *entire* data is read from the non-volatile       --
--  into memory. Successive reads will read from there.                      --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/01/27 15:23:48 $
--  $RCSfile: buffers-unbounded.ads,v $
--  $Revision: 1.1 $
--                                                                           --
-------------------------------------------------------------------------------

with Storages.Volatile.Memory_Storages;
use Storages.Volatile.Memory_Storages;

package Buffers.Unbounded is

   type Unbounded_Buffer_Type
      is new Buffer_Type with private;
   type Unbounded_Buffer_Ref is access all Unbounded_Buffer_Type'Class;

   procedure Read (Buffer : in out Unbounded_Buffer_Type;
                   Item    : out Stream_Element_Array;
                   Last    : out Stream_Element_Offset);
   --  Read a data item from the Buffer

   procedure Write (Buffer : in out Unbounded_Buffer_Type;
                    Item    : in Stream_Element_Array);
   --  Append 'Data' to the Buffer

   procedure Flush (Buffer : in out Unbounded_Buffer_Type);
   --  Write the buffered data to the associated storage

private

   type Unbounded_Buffer_Type is new Buffer_Type with record
      Memory_Stream : Memory_Storage_Ref;
   end record;

   procedure Finalize (Buffer : in out Unbounded_Buffer_Type);

end Buffers.Unbounded;



