-------------------------------------------------------------------------------
--                                  Buffers                                  --
--                                                                           --
--                  by Jrg Kienzle, last modified 28.4.1999                 --
--                                                                           --
--  The abstract type buffer type represents all possible sorts of buffers   --
--  that can be used in conjunction with streams.                            --
--  A buffer has an associated non-volatile storage, to which data will be   --
--  written when Flush is called. Note here that the data can also be        --
--  written to the storage under other conditions, for instance when the     --
--  buffer is full, or when a certain amount of data has been gathered.      --
-------------------------------------------------------------------------------

with Ada.Streams; use Ada.Streams;
with Ada.Finalization; use Ada.Finalization;
with Storages.Non_Volatile;
use  Storages.Non_Volatile;

package Buffers is

   type Buffer_Type (Storage : access Non_Volatile_Storage_Type'Class)
      is abstract new Limited_Controlled with private;

   type Buffer_Ref is access all Buffer_Type'Class;

   procedure Read (Buffer : in out Buffer_Type;
                   Item    : out Stream_Element_Array;
                   Last    : out Stream_Element_Offset) is abstract;
   --  Read a data item from the Buffer

   procedure Write (Buffer : in out Buffer_Type;
                    Item    : in Stream_Element_Array) is abstract;
   --  Append 'Data' to the Buffer

   procedure Flush (Buffer : in out Buffer_Type) is abstract;
   --  Write the buffered data to the associated storage

private

   type Buffer_Type (Storage : access Non_Volatile_Storage_Type'Class)
      is abstract new Limited_Controlled with null record;

   procedure Finalize (Buffer : in out Buffer_Type);

end Buffers;

