-------------------------------------------------------------------------------
--                                  Streams                                  --
--                                                                           --
--  The package streams presents the user with a concrete stream type.       --
--  (see ARM 13.13 for more detail). There are two version of streams,       --
--  normal streams, and buffered streams.                                    --
--                                                                           --
--  Normal streams have an associated storage, and they write the data to    --
--  that storage on every call to Write. When Read is called, the data is    --
--  read from the storage.                                                   --
--                                                                           --
--  Buffered streams are a little different. They write the data into an     --
--  intermediate (volatile) storage (a buffer), and only write the data to   --
--  the associated non-volatile storage under certain conditions (for        --
--  instance when the buffer is full.) To force the data to be written       --
--  to the non-volatile storage, Flush must be called.                       --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/01/05 12:57:06 $
--  $RCSfile: streams.adb,v $
--  $Revision: 1.1.1.1 $
--                                                                           --
-------------------------------------------------------------------------------

package body Streams is

   ----------------------------
   -- Stream_Type operations --
   ----------------------------

   procedure Read (Stream : in out Stream_Type;
                   Item   : out Stream_Element_Array;
                   Last   : out Stream_Element_Offset) is
   begin
      Read (Stream.Storage.all, Item, Last);
   end Read;

   procedure Write (Stream : in out Stream_Type;
                    Item   : in Stream_Element_Array) is
   begin
      Write (Stream.Storage.all, Item);
   end Write;

   -------------------------------------
   -- Buffered_Stream_Type operations --
   -------------------------------------

   procedure Read (Stream : in out Buffered_Stream_Type;
                   Item   : out Stream_Element_Array;
                   Last   : out Stream_Element_Offset) is
   begin
      Read (Stream.Buffer.all, Item, Last);
   end Read;

   procedure Write (Stream : in out Buffered_Stream_Type;
                    Item   : in Stream_Element_Array) is
   begin
      Write (Stream.Buffer.all, Item);
   end Write;

   procedure Flush (Stream : in out Buffered_Stream_Type) is
   begin
      Flush (Stream.Buffer.all);
   end Flush;

end Streams;
