-------------------------------------------------------------------------------
--                         Transaction OO Interface                          --
--                                                                           --
--  This package provides the interface to the transaction system            --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2001/03/27 09:58:15 $
--  $RCSfile: object_based_transaction_interface.adb,v $
--  $Revision: 1.1 $
--                                                                           --
-------------------------------------------------------------------------------

with Cache_Managers;
with Recovery_Support; use Recovery_Support;
with Transaction_Identifiers.Task_Attribute_Interface;
use Transaction_Identifiers.Task_Attribute_Interface;

package body Object_Based_Transaction_Interface is

   function Create (Params   : Non_Volatile_Params_Type'Class;
                    Creation : Creation_Operation_Type'Class;
                    Loading  : Loading_Operation_Type'Class;
                    Saving   : Saving_Operation_Type'Class;
                    Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref renames Transaction_Identifiers.
     Task_Attribute_Interface.Create;

   function Restore (Params   : Non_Volatile_Params_Type'Class;
                     Loading  : Loading_Operation_Type'Class;
                     Saving   : Saving_Operation_Type'Class;
                     Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref is
   begin
      return Cache_Managers.Restore (Get_Cache_Manager.all, Params,
                                     Loading, Saving, Deletion);
   end Restore;

   procedure System_Init renames Recovery_Support.System_Init;

   procedure System_Init (Anchor : in Stable_Params_Type'Class;
                          Params : in Stable_Params_Type'Class)
     renames Recovery_Support.System_Init;

   procedure System_Shutdown renames Recovery_Support.System_Shutdown;

   procedure Close_Transaction (T : in out Transaction) is
   begin
      Close_Transaction;
   end Close_Transaction;

   procedure Commit_Transaction (T : in out Transaction) is
   begin
      Commit_Transaction;
      T.Committed := True;
   end Commit_Transaction;

   procedure Initialize (T : in out Transaction) is
   begin
      if T.The_Task /= null then
         Join_Transaction (T.The_Task.all);
      elsif T.Name /= null then
         Begin_Or_Join_Transaction (T.Name.all);
      else
         Begin_Transaction;
      end if;
   end Initialize;

   procedure Finalize (T : in out Transaction) is
   begin
      if not T.Committed then
         Abort_Transaction;
      end if;
   end Finalize;

end Object_Based_Transaction_Interface;
