-------------------------------------------------------------------------------
--                         Transaction OO Interface                          --
--                                                                           --
--  This package provides the interface to the transaction system            --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2001/03/27 09:58:16 $
--  $RCSfile: object_based_transaction_interface.ads,v $
--  $Revision: 1.1 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Finalization; use Ada.Finalization;

with Transactions;
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;
with Storage_Params.Non_Volatile.Stable;
use Storage_Params.Non_Volatile.Stable;
with Memory_Objects; use Memory_Objects;
with Memory_Objects.Concrete; use Memory_Objects.Concrete;
with Operations; use Operations;
with Operations.Creation; use Operations.Creation;
with Operations.Loading; use Operations.Loading;
with Operations.Saving; use Operations.Saving;
with Operations.Deletion; use Operations.Deletion;
with Ada.Task_Identification; use Ada.Task_Identification;

package Object_Based_Transaction_Interface is

   type String_Ref is access all String;

   type Task_Id_Ref is access all Task_Id;

   type Transaction (Name : String_Ref := null;
                     The_Task : Task_Id_Ref := null;
                     Maximal_Participants : Natural := 0) is limited private;

   procedure Close_Transaction (T : in out Transaction);

   procedure Commit_Transaction (T : in out Transaction);

   No_Current_Transaction : exception
     renames Transactions.No_Current_Transaction;
   Already_Joined_Transaction : exception
     renames Transactions.No_Current_Transaction;
   Already_Part_Of_Other_Transaction : exception
     renames Transactions.No_Current_Transaction;
   Transaction_Already_Committed : exception
     renames Transactions.Transaction_Already_Committed;
   Transaction_Aborted : exception
     renames Transactions.Transaction_Aborted;
   Transaction_Closed : exception
     renames Transactions.Transaction_Closed;
   Transaction_Not_Joined : exception;

   function Create (Params   : Non_Volatile_Params_Type'Class;
                    Creation : Creation_Operation_Type'Class;
                    Loading  : Loading_Operation_Type'Class;
                    Saving   : Saving_Operation_Type'Class;
                    Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref;

   function Restore (Params   : Non_Volatile_Params_Type'Class;
                     Loading  : Loading_Operation_Type'Class;
                     Saving   : Saving_Operation_Type'Class;
                     Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref;

   Object_Exists : exception
     renames Memory_Objects.Concrete.Object_Exists;
   Object_Non_Existant : exception
     renames Memory_Objects.Concrete.Object_Non_Existant;
   Object_Corrupt : exception
     renames Memory_Objects.Concrete.Object_Corrupt;

   procedure System_Init;

   procedure System_Init (Anchor : in Stable_Params_Type'Class;
                          Params : in Stable_Params_Type'Class);

   procedure System_Shutdown;

private

   type Transaction (Name : String_Ref := null;
                     The_Task : Task_Id_Ref := null;
                     Maximal_Participants : Natural := 0)
   is new Ada.Finalization.Limited_Controlled with record
      Committed : Boolean := False;
   end record;

   procedure Initialize (T : in out Transaction);

   procedure Finalize (T : in out Transaction);

end Object_Based_Transaction_Interface;
