-------------------------------------------------------------------------------
--                         Transaction Interface                             --
--                                                                           --
--  This package provides the interface to the transaction system            --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2001/04/23 11:19:13 $
--  $RCSfile: procedural_transaction_interface.adb,v $
--  $Revision: 1.2 $
--                                                                           --
-------------------------------------------------------------------------------

with Cache_Managers;
with Recovery_Support; use Recovery_Support;
with Transaction_Identifiers.Task_Attribute_Interface;

package body Procedural_Transaction_Interface is

   procedure Begin_Transaction renames Transaction_Identifiers.
     Task_Attribute_Interface.Begin_Transaction;

   procedure Join_Transaction (The_Task : Task_Id)
     renames Transaction_Identifiers.
     Task_Attribute_Interface.Join_Transaction;

   procedure Begin_Or_Join_Transaction (Name : String) renames
     Transaction_Identifiers.
     Task_Attribute_Interface.Begin_Or_Join_Transaction;

   procedure Close_Transaction renames Transaction_Identifiers.
     Task_Attribute_Interface.Close_Transaction;

   procedure Abort_Transaction renames Transaction_Identifiers.
     Task_Attribute_Interface.Abort_Transaction;

   procedure Commit_Transaction renames Transaction_Identifiers.
     Task_Attribute_Interface.Commit_Transaction;

   function Create (Params   : Non_Volatile_Params_Type'Class;
                    Creation : Creation_Operation_Type'Class;
                    Loading  : Loading_Operation_Type'Class;
                    Saving   : Saving_Operation_Type'Class;
                    Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref renames Transaction_Identifiers.
     Task_Attribute_Interface.Create;

   function Restore (Params   : Non_Volatile_Params_Type'Class;
                     Loading  : Loading_Operation_Type'Class;
                     Saving   : Saving_Operation_Type'Class;
                     Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref is
   begin
      return Cache_Managers.Restore (Get_Cache_Manager.all, Params,
                                     Loading, Saving, Deletion);
   end Restore;

   procedure System_Init renames Recovery_Support.System_Init;

   procedure System_Init (Anchor : in Stable_Params_Type'Class;
                          Params : in Stable_Params_Type'Class)
     renames Recovery_Support.System_Init;

   procedure System_Shutdown renames Recovery_Support.System_Shutdown;

end Procedural_Transaction_Interface;
