-------------------------------------------------------------------------------
--                         Transaction Interface                             --
--                                                                           --
--  This package provides the interface to the transaction system            --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2001/04/23 11:19:13 $
--  $RCSfile: procedural_transaction_interface.ads,v $
--  $Revision: 1.2 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Task_Identification; use Ada.Task_Identification;

with Transactions;
with Storage_Params.Non_Volatile; use Storage_Params.Non_Volatile;
with Storage_Params.Non_Volatile.Stable;
use Storage_Params.Non_Volatile.Stable;
with Memory_Objects; use Memory_Objects;
with Memory_Objects.Concrete; use Memory_Objects.Concrete;
with Operations; use Operations;
with Operations.Creation; use Operations.Creation;
with Operations.Loading; use Operations.Loading;
with Operations.Saving; use Operations.Saving;
with Operations.Deletion; use Operations.Deletion;

package Procedural_Transaction_Interface is

   procedure Begin_Transaction;

   procedure Join_Transaction (The_Task : Task_Id);

   procedure Begin_Or_Join_Transaction (Name : String);

   procedure Close_Transaction;

   procedure Abort_Transaction;

   procedure Commit_Transaction;

   No_Current_Transaction : exception
     renames Transactions.No_Current_Transaction;
   Already_Joined_Transaction : exception
     renames Transactions.No_Current_Transaction;
   Already_Part_Of_Other_Transaction : exception
     renames Transactions.No_Current_Transaction;
   Transaction_Already_Committed : exception
     renames Transactions.Transaction_Already_Committed;
   Transaction_Aborted : exception
     renames Transactions.Transaction_Aborted;

   function Create (Params   : Non_Volatile_Params_Type'Class;
                    Creation : Creation_Operation_Type'Class;
                    Loading  : Loading_Operation_Type'Class;
                    Saving   : Saving_Operation_Type'Class;
                    Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref;

   function Restore (Params   : Non_Volatile_Params_Type'Class;
                     Loading  : Loading_Operation_Type'Class;
                     Saving   : Saving_Operation_Type'Class;
                     Deletion : Deletion_Operation_Type'Class)
     return Memory_Object_Ref;

   Object_Exists : exception
     renames Memory_Objects.Concrete.Object_Exists;
   Object_Non_Existant : exception
     renames Memory_Objects.Concrete.Object_Non_Existant;
   Object_Corrupt : exception
     renames Memory_Objects.Concrete.Object_Corrupt;

   procedure System_Init;

   procedure System_Init (Anchor : in Stable_Params_Type'Class;
                          Params : in Stable_Params_Type'Class);

   procedure System_Shutdown;

end Procedural_Transaction_Interface;
