-------------------------------------------------------------------------------
--             Transaction Identifiers - Task Attribute Interface            --
--                                                                           --
--  This package provides the bridge between the task attributes and the     --
--  task identifiers                                                         --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2001/03/27 09:58:16 $
--  $RCSfile: transaction_identifiers-task_attribute_interface.ads,v $
--  $Revision: 1.5 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Task_Identification; use Ada.Task_Identification;

with Transactions; use Transactions;
with Memory_Objects; use Memory_Objects;
with Storage_Params.Non_Volatile;
use Storage_Params.Non_Volatile;
with Operations.Creation; use Operations.Creation;
with Operations.Loading; use Operations.Loading;
with Operations.Saving; use Operations.Saving;
with Operations.Deletion; use Operations.Deletion;

package Transaction_Identifiers.Task_Attribute_Interface is

   procedure Begin_Transaction;

   procedure Join_Transaction (The_Task : Task_Id);

   procedure Join_Transaction (Transaction_Id : Transaction_Identifier_Type);

   procedure Begin_Or_Join_Transaction (Name : String);

   procedure Close_Transaction;

   procedure Abort_Transaction;

   procedure Commit_Transaction;

   function Create (Params   : Non_Volatile_Params_Type'Class;
                    Creation : Creation_Operation_Type'Class;
                    Loading  : Loading_Operation_Type'Class;
                    Saving   : Saving_Operation_Type'Class;
                    Deletion : Deletion_Operation_Type'Class)
      return Memory_Object_Ref;

   function Get_Current_Transaction_Id return Transaction_Identifier_Type;

   function Get_Current_Transaction return Transaction_Ref;

end Transaction_Identifiers.Task_Attribute_Interface;
