-------------------------------------------------------------------------------
--                         Transaction Identifiers                           --
--                                                                           --
--  This package provides transaction identifiers                            --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/08/18 11:48:54 $
--  $RCSfile: transaction_identifiers.adb,v $
--  $Revision: 1.5 $
--                                                                           --
-------------------------------------------------------------------------------

with Transactions; use Transactions;

package body Transaction_Identifiers is

   function Get_New_Transaction_Id return Transaction_Identifier_Type is
   begin
      return Transaction_Identifier_Type' (Ada.Finalization.Controlled with
         Transaction => Create_New_Toplevel_Transaction);
   end Get_New_Transaction_Id;

   function Get_Nested_Transaction_Id (Parent : Transaction_Identifier_Type)
      return Transaction_Identifier_Type is
      Temp : Transaction_Ref;
   begin
      Create_Nested_Transaction (Parent.Transaction.all, Temp);
      return Transaction_Identifier_Type' (Ada.Finalization.Controlled with
         Transaction => Temp);
   end Get_Nested_Transaction_Id;

   function Get_Parent_Transaction_Id (Id : Transaction_Identifier_Type)
      return Transaction_Identifier_Type is
   begin
      if Is_Toplevel (Id.Transaction.all) then
         return Null_Transaction_Id;
      else
         declare
            Parent : Transaction_Ref := Get_Parent (Id.Transaction.all);
         begin
            Increase_Reference_Count (Parent);
            return Transaction_Identifier_Type' (Ada.Finalization.Controlled
                                                 with Transaction => Parent);
         end;
      end if;
   end Get_Parent_Transaction_Id;

   procedure Adjust (Id : in out Transaction_Identifier_Type) is
   begin
      if Id.Transaction /= null then
         Increase_Reference_Count (Id.Transaction);
      end if;
   end Adjust;

   procedure Finalize (Id : in out Transaction_Identifier_Type) is
   begin
      if Id.Transaction /= null then
         Decrease_Reference_Count (Id.Transaction);
         Check_Reference_Count (Id.Transaction);
      end if;
   end Finalize;

end Transaction_Identifiers;


