-------------------------------------------------------------------------------
--                         Transaction Identifiers                           --
--                                                                           --
--  This package provides transaction identifiers                            --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/08/18 11:48:55 $
--  $RCSfile: transaction_identifiers.ads,v $
--  $Revision: 1.5 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Finalization;
with Transactions; use Transactions;

package Transaction_Identifiers is

   type Transaction_Identifier_Type is private;
   --  This is the TID type.
   --  It contains a list of subtransactions.
   type Transaction_Identifier_Ref is access all Transaction_Identifier_Type;

   Null_Transaction_Id : constant Transaction_Identifier_Type;

   function Get_New_Transaction_Id return Transaction_Identifier_Type;

   function Get_Nested_Transaction_Id (Parent : Transaction_Identifier_Type)
      return Transaction_Identifier_Type;

   function Get_Parent_Transaction_Id (Id : Transaction_Identifier_Type)
      return Transaction_Identifier_Type;

private

   type Transaction_Identifier_Type is new Ada.Finalization.Controlled
   with record
      Transaction : Transaction_Ref := null;
   end record;

   procedure Adjust (Id : in out Transaction_Identifier_Type);

   procedure Finalize (Id : in out Transaction_Identifier_Type);

   Null_Transaction_Id : constant Transaction_Identifier_Type :=
     (Ada.Finalization.Controlled with Transaction => null);

end Transaction_Identifiers;


