-------------------------------------------------------------------------------
--                               Keyed Lists                                 --
--                                                                           --
--  This is a utility package to create keyed lists, mainly used for         --
--  transaction lists...                                                     --
--                                                                           --
--  $Author: xcaron $
--  $Date: 2000/11/30 12:26:18 $
--  $RCSfile: keyed_lists.ads,v $
--  $Revision: 1.5 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Finalization; use Ada.Finalization;

generic

   type Key_Type is private;

   type Element_Type is private;

   type Additional_Info_Type is private;

package Keyed_Lists is

   type List_Type is limited private;

   type Element_Node_Type;

   type Element_Node_Ref is access Element_Node_Type;

   type Element_Node_Type is record
      Next : Element_Node_Ref;
      Element : Element_Type;
   end record;

   function Find (List : List_Type;
                  Key : Key_Type) return Element_Node_Ref;

   procedure Create_Key (List : in out List_Type;
                         Key : in Key_Type;
                         Info : in Additional_Info_Type);

   procedure Replace_Key (List : in out List_Type;
                          Key : in Key_Type;
                          New_Key : in Key_Type);

   procedure Set_Info (List : in out List_Type;
                       Key : in Key_Type;
                       Info : in Additional_Info_Type);

   function Get_Info (List : List_Type;
                      Key : Key_Type) return Additional_Info_Type;

   procedure Insert (List : in out List_Type;
                     Key : in Key_Type;
                     Elements : in Element_Node_Ref);

   procedure Merge_Lists (List : in out List_Type;
                          Dest_Key : in Key_Type;
                          Source_Key : in Key_Type);

   procedure Delete (List : in out List_Type;
                     Key : in Key_Type);

   procedure Insert_Element (List : in out List_Type;
                             Key : in Key_Type;
                             Element : in Element_Type;
                             Create_Key : in Boolean := False);

   procedure Append_Element (List : in out List_Type;
                             Key : in Key_Type;
                             Element : in Element_Type);

   procedure Delete_Element (List : in out List_Type;
                             Key : in Key_Type;
                             Element : in Element_Type);

   No_Such_Key, No_Such_Element, Key_Exists : exception;

   generic
      with procedure Action (Element : in out Key_Type);
   procedure Iterate_Through_Keys (List : in out List_Type);

private

   type Key_Node_Type;

   type Key_Node_Ref is access Key_Node_Type;

   type Key_Node_Type is record
      Next : Key_Node_Ref := null;
      Key : Key_Type;
      Info : Additional_Info_Type;
      Elements : Element_Node_Ref := null;
   end record;

   type List_Type is new Limited_Controlled with record
      First : Key_Node_Ref := null;
   end record;

   procedure Finalize (List : in out List_Type);

end Keyed_Lists;
