with Ada.Unchecked_Deallocation;

package body Linked_Lists is

   procedure Free is new Ada.Unchecked_Deallocation
     (List_Node_Type, List_Type);

   function Count_Elements (List : List_Type) return Natural is
      N : Natural := 0;
      Tmp : List_Type := List;
   begin
      while Tmp /= null loop
         N := N + 1;
         Tmp := Tmp.Next;
      end loop;
      return N;
   end Count_Elements;

   procedure Insert_Element (List : in out List_Type;
                             Element : in Element_Type) is
      New_Node : List_Type := new List_Node_Type;
   begin
      New_Node.Next := List;
      New_Node.Element := Element;
      List := New_Node;
   end Insert_Element;

   procedure Insert_If_Not_Already (List : in out List_Type;
                                    Element : in Element_Type) is
      Tmp : List_Type := List;
   begin
      while Tmp /= null loop
         if Tmp.Element = Element then
            return;
         end if;
         Tmp := Tmp.Next;
      end loop;
      Tmp := new List_Node_Type;
      Tmp.Next := List;
      Tmp.Element := Element;
      List := Tmp;
   end Insert_If_Not_Already;

   procedure Merge_Lists (List : in out List_Type;
                          List_To_Merge : in List_Type) is
      Tmp : List_Type := List_To_Merge;
   begin
      while Tmp /= null loop
         Insert_If_Not_Already (List, Tmp.Element);
         Tmp := Tmp.Next;
      end loop;
   end Merge_Lists;

   procedure Delete_Element (List : in out List_Type;
                             Element : in Element_Type) is
      Tmp : List_Type := List;
      Prev : List_Type := null;
   begin
      while Tmp /= null loop
         if Tmp.Element = Element then
            if Prev = null then
               List := Tmp.Next;
            else
               Prev.Next := Tmp.Next;
            end if;
            Free (Tmp);
            return;
         else
            Prev := Tmp;
            Tmp := Tmp.Next;
         end if;
      end loop;
      raise No_Such_Element;
   end Delete_Element;

   procedure Delete_List (List : in out List_Type) is
      Tmp : List_Type := List;
      To_Free : List_Type := null;
   begin
      while Tmp /= null loop
         To_Free := Tmp;
         Tmp := Tmp.Next;
         Free (To_Free);
      end loop;
      List := null;
   end Delete_List;

end Linked_Lists;
