-------------------------------------------------------------------------------
--                             Refkeyed Lists                                --
--                                                                           --
--  This is a utility package to create keyed lists, but thistime the key    --
--  is referenced                                                            --
--                                                                           --
--  $Author: jkienzle $
--  $Date: 2000/09/29 07:18:28 $
--  $RCSfile: refkeyed_lists.ads,v $
--  $Revision: 1.2 $
--                                                                           --
-------------------------------------------------------------------------------

with Ada.Finalization; use Ada.Finalization;

generic

   type Key_Type is abstract tagged private;

   type Key_Ref is access Key_Type'Class;

   type Element_Type is private;

   type Additional_Info_Type is private;

   with function "<" (Left, Right : Additional_Info_Type)
      return Boolean is <>;

package Refkeyed_Lists is

   type List_Type is limited private;

   type Element_Node_Type;

   type Element_Node_Ref is access Element_Node_Type;

   type Element_Node_Type is record
      Next : Element_Node_Ref;
      Element : Element_Type;
   end record;

   function Find (List : List_Type;
                  Key : Key_Ref) return Element_Node_Ref;

   function Find (List : List_Type;
                  Key : Key_Type'Class) return Element_Node_Ref;

   procedure Create_Key (List : in out List_Type;
                         Key : in Key_Ref;
                         Info : in Additional_Info_Type);

   procedure Set_Info (List : in out List_Type;
                       Key : in Key_Ref;
                       Info : in Additional_Info_Type);

   function Get_Info (List : List_Type;
                      Key : Key_Ref) return Additional_Info_Type;

   function Get_Info (List : List_Type;
                      Key : Key_Type'Class) return Additional_Info_Type;

   procedure Insert (List : in out List_Type;
                     Key : in Key_Ref;
                     Elements : in Element_Node_Ref);

   procedure Insert_Or_Prepend (List : in out List_Type;
                                Key : in Key_Ref;
                                Elements : in Element_Node_Ref);

   procedure Delete (List : in out List_Type;
                     Key : in Key_Ref);

   procedure Insert_Element (List : in out List_Type;
                             Key : in Key_Ref;
                             Element : in Element_Type;
                             Create_Key : in Boolean := False);

   procedure Insert_Element_With_Info (List : in out List_Type;
                                       Key : in Key_Ref;
                                       Element : in Element_Type;
                                       Info : in Additional_Info_Type;
                                       Create_Key : in Boolean := False);

   procedure Append_Element (List : in out List_Type;
                             Key : in Key_Ref;
                             Element : in Element_Type);

   procedure Delete_Element (List : in out List_Type;
                             Key : in Key_Ref;
                             Element : in Element_Type);

   generic
      with procedure Action (Key : in Key_Ref);
   procedure Iterate_Through_Keys (List : List_Type);

   No_Such_Key, No_Such_Element, Key_Exists : exception;

private

   type Key_Node_Type;

   type Key_Node_Ref is access Key_Node_Type;

   type Key_Node_Type is record
      Next : Key_Node_Ref;
      Key : Key_Ref;
      Info : Additional_Info_Type;
      Elements : Element_Node_Ref;
   end record;

   type List_Type is new Limited_Controlled with record
      First : Key_Node_Ref;
   end record;

   procedure Finalize (List : in out List_Type);

end Refkeyed_Lists;
